#!/bin/bash


cd $( dirname $0 )

## Set defaults
##---------------------------------------------------
DBG=0; PROD=0; HELP=0
TRUS_DIR=$PWD; TRUS_RSLT='FAILED'; SVN_CMD='svn status' # ('FAILED' result & no update on directories)
xios_mode='--full'; stdout_redir='>&'
revi=$( svn info | awk '(NR == 9) {print $NF}' )


## Get options for replacing initials settings
##---------------------------------------------------
while [ $# -ne 0 ]; do
    case $1 in
	'-a'|'--archive') TRUS_TARF=$2  ; shift 2;; '-b'|'--branch' ) TRUS_WKCY=$2; shift 2;;
	'-d'|'--debug'  ) set -vx; DBG=1; shift  ;; '-e'|'--email'  ) TRUS_MAIL=$2; shift 2;;
	'-j'|'--job'    ) TRUS_NPRO=$2  ; shift 2;; '-h'|'--help'   ) HELP=1      ; shift  ;;
	'-m'|'--machine') TRUS_HPCC=$2  ; shift 2;; '-n'|'--newconf') TRUS_TEST=$2; shift 2;;
	'-r'|'--refconf') TRUS_REFE=$2  ; shift 2;; '-t'|'--time'   ) TIMEOUT=$2  ; shift 2;;
	'-p'|'--prod'   ) PROD=1        ; shift  ;; '-u'|'--user'   ) TRUS_USER=$2; shift 2;;
	'-v'|'--version') TRUS_VERS=$2  ; shift 2;; "*"             ) HELP=1      ; shift  ;;
    esac
done


## Initialization (HPC & user environment)
##---------------------------------------------------
if [[ ! $( find config -name ${TRUS_USER}'.cfg' && find config -name ${TRUS_HPCC}'.cfg' ) || $HELP -eq 1 ]]; then
    cat trusting_help.txt

    if [ $HELP -eq 0 ]; then
	printf "\n\nAt least one configuration file is missing or misspelled: %s.cfg %s.cfg" \
	    ${TRUS_USER} ${TRUS_HPCC}
    fi

    printf "\n\nContent of config folder:"
    find config -name *.cfg | cut -d/ -f2 | xargs -n 3 printf "%-30s\t%-30s\t%-30s\n"
    exit 1
else
    . trusting.env && . trusting_func.sh

    ## DEBUG options to speed up & expand verbosity of compilation
    [ $DBG -eq 1 ] && { xios_mode=''; stdout_redir='>'; }

    ## If -v|--version option has been set, modify default SVN action on directories
    if   [ $( echo ${TRUS_VERS} | grep  "HEAD\|up\|update"                     ) ]; then
	SVN_CMD='svn update -r HEAD'
    elif [ $( echo ${TRUS_VERS} | grep -o '{[0-9]\{4\}-[0-9]\{2\}-[0-9]\{2\}}' ) ]; then
	SVN_CMD='svn update -r '$( echo ${TRUS_VERS} | grep -o '{[0-9]\{4\}-[0-9]\{2\}-[0-9]\{2\}}' )
    elif [ $( echo ${TRUS_VERS} | grep -o '[0-9]*'                             ) ]; then
	SVN_CMD='svn update -r '$( echo ${TRUS_VERS} | grep -o '[0-9]*'                             )
    fi

fi


## Display contextual summary of trusting test
##---------------------------------------------------
echo
cat banner.txt
echo
echo '****************************************************************************************************'
echo '*                                                                                                  *'
echo '*                           NEMO Trusting (Continuous Integration Tool)                            *'
echo "*                                             ver.$revi                                             *"
echo '*                                                                                                  *'
echo '****************************************************************************************************'
echo
echo '  - Testing configuration   '${TRUS_TEST}' based on '${TRUS_REFE}
echo '  - Working copy            '${TRUS_WKCY}
echo '  - Benchmark folder        '${TRUS_BHMK}
echo '  - (Super)Computer         '${TRUS_HPCC}
echo '  - User installation       '${TRUS_USER}
echo


## Make timestamped directory with messenger files
##---------------------------------------------------
print_step 'Timestamped testing directory'
mkdir -p ${TEST_DIR} ${TRUS_BHMK}
cd ${TEST_DIR} && echo ${TEST_DIR}
init_files
get_date


## Get SVN revision on XIOS & NEMO essentials directories
##---------------------------------------------------
print_step "SVN action on NEMO directories: ${SVN_CMD}"
get_nemo_rev


## Check softwares versions (after sourced arch environment)
##---------------------------------------------------
print_step 'Get testing environement'
get_soft_rel
cat model.log
env | sort > env.log


## XIOS compilation from scratch
##---------------------------------------------------
print_step 'Compile XIOS'
cd ${DIR_XIOS}
eval ./make_xios ${xios_mode} --arch ${TRUS_HPCC} --job ${TRUS_NPRO} \
    ${stdout_redir} /dev/null
[ ! -e lib/libxios.a ] && get_out 1 || echo 'Success'


## NEMO compilation from scratch
##---------------------------------------------------
print_step "Compile ${TRUS_REFE} configuration"
cd ${TRUS_WKCY}/CONFIG
[[ -d ${TRUS_TEST} && $DBG -eq 0 ]] && ./makenemo -n ${TRUS_TEST} clean_config \
    > /dev/null <<EOF
y
EOF

eval ./makenemo -n ${TRUS_TEST} -r ${TRUS_REFE} -m ${TRUS_HPCC} -j ${TRUS_NPRO} \
                   ${KEYS_ADD} ${KEYS_DEL}                                      \
    ${stdout_redir} /dev/null
[ ! -e ${TRUS_TEST}/BLD/bin/nemo.exe ] && get_out 2 || echo 'Success'


## Get all inputs for running
##---------------------------------------------------
print_step 'Set job (copying or extracting inputs)'
cd ${TEST_DIR}
cp   ${TRUS_WKCY}/CONFIG/${TRUS_TEST}/cpp_* .
find ${TRUS_WKCY}/CONFIG/${TRUS_TEST}/EXP00 -regex '.*\(_cfg\|.in\|opa\|_ref\|.xml\)' \
                                            -exec  cp {} . \;
get_inputs
[ $? -ne 0 ] && get_out 3 || echo 'Success'
[ $( find . -name '*.gz' -print -quit ) ] && find . -name '*.gz' -exec gunzip {} \;


## Check inputs
##---------------------------------------------------
print_step 'Compare inputs'
diff_inputs


## Job submission & computation
##---------------------------------------------------
print_step 'Submit job'
JOB_ID=$( eval ${JOB_SUBM} )
[ $? -ne 0 ] && get_out 4 || echo 'Success (job ID '${JOB_ID}')'
print_step 'Pending job'
job_pending
print_step 'Job finished'


## Check job state & get computation performances if succeeded
##---------------------------------------------------
print_step 'Test job state'
[[ ! -e time.step || $( grep 'E R R O R' ocean.output ) ]] && get_out 5 || echo 'Success'
print_step 'Get job performances'
get_time
get_memy


## Check outputs
##---------------------------------------------------
TRUS_RSLT='OK' ## 'OK' by default
print_step 'Compare outputs'
diff_results
print_step 'Compare restarts'
diff_restart
[ $TRUS_RSLT == 'FAILED' ] && get_out 8


## End, at least nothing has changed ;-)
##---------------------------------------------------
get_out 0
